package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluateRule;
import com.ims.eval.entity.IndicatorType;
import com.ims.eval.entity.OrganizationEvaluationRule;
import com.ims.eval.dao.OrganizationEvaluationRuleMapper;
import com.ims.eval.entity.dto.request.OrganizationEvaluationRuleDTO;
import com.ims.eval.service.IEvaluateRuleService;
import com.ims.eval.service.IOrganizationEvaluationRuleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 单位/部门考评配置 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-01
 */
@Service
public class OrganizationEvaluationRuleServiceImpl extends ServiceImpl<OrganizationEvaluationRuleMapper, OrganizationEvaluationRule> implements IOrganizationEvaluationRuleService {


	@Autowired
	private IEvaluateRuleService evaluateRuleService;


	@Override
	public IPage<OrganizationEvaluationRule> list(Integer pageNum, Integer pageSize, String id, String organizationName, String organizationId, String organizationType, String binSection, String binStage, String evaluationCycle) {
		QueryWrapper<OrganizationEvaluationRule> qw = new QueryWrapper<>();
		//构造分页构造器
		Page<OrganizationEvaluationRule> page = new Page<>(pageNum, pageSize);
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(OrganizationEvaluationRule::getId, id);
		}

		if (StringUtils.isNotEmpty(organizationName)) {
			qw.lambda().like(OrganizationEvaluationRule::getOrganizationName, organizationName);
		}

		if (StringUtils.isNotEmpty(organizationId)) {
			qw.lambda().like(OrganizationEvaluationRule::getOrganizationId, organizationId);
		}

		if (StringUtils.isNotEmpty(organizationType)) {
			qw.lambda().eq(OrganizationEvaluationRule::getOrganizationType, organizationType);
		}

		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(OrganizationEvaluationRule::getBinSection, binSection);
		}

		if (StringUtils.isNotEmpty(binStage)) {
			qw.lambda().eq(OrganizationEvaluationRule::getBinStage, binStage);
		}

		if (StringUtils.isNotEmpty(evaluationCycle)) {
			qw.lambda().eq(OrganizationEvaluationRule::getEvaluationCycle, evaluationCycle);
		}


		qw.lambda().eq(OrganizationEvaluationRule::getDelFlag, false);


		qw.lambda().orderByAsc(OrganizationEvaluationRule::getOrderNum);

		IPage<OrganizationEvaluationRule> list = baseMapper.selectPage(page, qw);

		list.getRecords().stream().forEach(l->{
			l.setBinSectionName(null == CacheContext.ddNameMap.get(l.getBinSection()) ? "" :CacheContext.ddNameMap.get(l.getBinSection()));
		});


		return list;
	}

	@Override
	public List<OrganizationEvaluationRule> listAll(String id, String organizationName, String organizationId, String organizationType, String binSection, String binStage, String evaluationCycle) {
		QueryWrapper<OrganizationEvaluationRule> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(OrganizationEvaluationRule::getId, id);
		}

		if (StringUtils.isNotEmpty(organizationName)) {
			qw.lambda().like(OrganizationEvaluationRule::getOrganizationName, organizationName);
		}

		if (StringUtils.isNotEmpty(organizationId)) {
			qw.lambda().like(OrganizationEvaluationRule::getOrganizationId, organizationId);
		}

		if (StringUtils.isNotEmpty(organizationType)) {
			qw.lambda().eq(OrganizationEvaluationRule::getOrganizationType, organizationType);
		}

		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(OrganizationEvaluationRule::getBinSection, binSection);
		}

		if (StringUtils.isNotEmpty(binStage)) {
			qw.lambda().eq(OrganizationEvaluationRule::getBinStage, binStage);
		}

		if (StringUtils.isNotEmpty(evaluationCycle)) {
			qw.lambda().eq(OrganizationEvaluationRule::getEvaluationCycle, evaluationCycle);
		}

		qw.lambda().eq(OrganizationEvaluationRule::getDelFlag, false);

		qw.lambda().orderByAsc(OrganizationEvaluationRule::getOrderNum);

		List<OrganizationEvaluationRule> list = baseMapper.selectList(qw);

		list.stream().forEach(l->{
			l.setBinSectionName(null == CacheContext.ddNameMap.get(l.getBinSection()) ? "" :CacheContext.ddNameMap.get(l.getBinSection()));
		});

		return list;
	}

	@Override
	public IPage<OrganizationEvaluationRule> list2(Integer pageNum, Integer pageSize, String id, String organizationName, String organizationId, String organizationType, String binSection, String binStage, String evaluationCycle) {
		Page<OrganizationEvaluationRule> page = new Page<>(pageNum, pageSize);
		IPage<OrganizationEvaluationRule>  list = baseMapper.list(page, id, organizationName, organizationId, organizationType, binSection, binStage, evaluationCycle);
		list.getRecords().stream().forEach(l->{
			l.setBinSectionName(null == CacheContext.ddNameMap.get(l.getBinSection()) ? "" :CacheContext.ddNameMap.get(l.getBinSection()));

			StringBuilder binStageName =  new StringBuilder();
			for (String stage : l.getBinStage().split(",")){
				binStageName.append(null == CacheContext.ddNameMap.get(stage) ? "" :CacheContext.ddNameMap.get(stage)).append(",");
			}

			l.setBinStageName(binStageName.toString().length()>0?binStageName.toString().substring(0,binStageName.toString().length()-1):"");

			StringBuilder ruleName = new StringBuilder();
			for (String ruleId :l.getEvaluateRuleId().split(",")){
				ruleName.append(evaluateRuleService.getById(ruleId).getRuleName()).append(",");
			}
			l.setEvaluateRuleName(ruleName.toString().length()>0?ruleName.toString().substring(0,ruleName.toString().length()-1):"");

		});
		return list;
	}


	@Transactional
	@Override
	public boolean saveOrUpdate(OrganizationEvaluationRule entity) {
		if (null != entity && (null == entity.getId() || "".equals(entity.getId().trim()))) {
			QueryWrapper<OrganizationEvaluationRule> qw = new QueryWrapper<>();
			qw.lambda().eq(OrganizationEvaluationRule::getOrganizationId, entity.getOrganizationId());
			qw.lambda().eq(OrganizationEvaluationRule::getEvaluationCycle, entity.getEvaluationCycle());
			List<OrganizationEvaluationRule> list = baseMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				throw new CustomException("已存在同期考评配置");
			}
			entity.setCreateTime(new Date());
		} else {
			entity.setUpdateTime(new Date());
		}

		return super.saveOrUpdate(entity);
	}

	@Transactional
	@Override
	public boolean removeByIds(Collection<? extends Serializable> idList) {

		for (Serializable ids  :idList){
			OrganizationEvaluationRule rule = baseMapper.selectById(ids);
			if(null == rule){
				throw new CustomException("查无该记录");
			}
			rule.setDelFlag(true);
			rule.setUpdateTime(new Date());
			int num = baseMapper.updateById(rule);
			if (num<=0){
				throw new CustomException("删除失败");
			}
		}
		return true;
	}
}
